# -*- coding: utf-8 -*-
"""
/***************************************************************************
 CtrlCraft
                                 A QGIS plugin
 Areva - plugin for displaying 3D mines
                              -------------------
        begin                : 2021-06-07
        git sha              : $Format:%H$
        copyright            : (C) 2021 by Sebastien Peillet (Oslandia)
        email                : contact@oslandia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os
import re
import xml.etree.ElementTree as ET
from PyQt5.QtWidgets import (
                             QLabel,
                             QWidget,
                             QHBoxLayout,
                             QVBoxLayout,
                             QDialog,
                             QPushButton,
                             QCheckBox
                            )
from PyQt5.Qt import QColor
from qgis.core import QgsProject
from qgis.gui import QgsDoubleSpinBox

class CtrlCraftSlabThreshold(QDialog):
    """Dialog used to define slab symbology threshold"""
    def __init__(self, project, data, parent=None):
        """Constructor of the dialog used to define slab symbology threshold"""
        super().__init__(parent)

        self.__project = project
        self.__data = data
        self.setWindowTitle(self.tr("Slab threshold"))

        self.__slabs_defaut_style_path = os.path.join(os.path.dirname(__file__),'..','qgis/slabs_defaut.qml')
        self.__probe_defaut_style_path = os.path.join(os.path.dirname(__file__),'..','qgis/probe_defaut.qml')
        self.__estimatedblocs_defaut_style_path = os.path.join(os.path.dirname(__file__),'..','qgis/estimatedblocs_defaut.qml')
        self.__atlas_style_path = os.path.join(os.path.dirname(__file__),'..','qgis/atlas.qml')

        self.__slabs_default_tree, self.__slabs_default_rule_dict = self.get_threshold_values(self.__slabs_defaut_style_path)
        self.__probe_default_tree, self.__probe_default_rule_dict = self.get_threshold_values(self.__probe_defaut_style_path)
        self.__estimatedblocs_default_tree, self.__estimatedblocs_default_rule_dict = self.get_threshold_values(self.__estimatedblocs_defaut_style_path)
        self.__atlas_tree, self.__atlas_rule_dict = self.get_threshold_values(self.__atlas_style_path)

        v_layout = QVBoxLayout(self)

        p = re.compile(r'\d+[\.?]\d+')
        for label, value in self.__slabs_default_rule_dict.items():
            h_layout = QHBoxLayout()
            h_layout.setObjectName(label)
            #from the rule label, get the label class
            label = QLabel()
            label.setMinimumWidth(25)
            label.setStyleSheet(f"background-color: rgb({value[2]})")
            #from the rule filter, get the min and max threshold
            minv, maxv = p.findall(value[1])[:2]
            min_spinbox = QgsDoubleSpinBox()
            min_spinbox.setDecimals(6)
            min_spinbox.setSingleStep(0.1)
            min_spinbox.setMinimum(0)
            min_spinbox.setMaximum(1000)
            min_spinbox.setValue(float(minv))
            max_spinbox = QgsDoubleSpinBox()
            max_spinbox.setDecimals(6)
            min_spinbox.setSingleStep(0.1)
            max_spinbox.setMinimum(0)
            max_spinbox.setMaximum(1000)
            max_spinbox.setValue(float(maxv))
            checkbox = QCheckBox()
            checkbox.setChecked(bool(value[3]))
            h_layout.addWidget(checkbox)
            h_layout.addWidget(label)
            h_layout.addWidget(min_spinbox)
            h_layout.addWidget(max_spinbox)
            v_layout.addLayout(h_layout)


        accept_button = QPushButton("OK")
        accept_button.clicked.connect(self.accept)
        v_layout.addWidget(accept_button)

    @staticmethod
    def get_threshold_values(path):
        """From a qml file,Return the xml tree of rules and a dictionnary compose as follow : {rule_label : [rule_xml_element, rule_filter, rule_symbol, rule_checked]}"""
        tree = ET.parse(path)
        root = tree.getroot()
        renderer = root.find('renderer-v2')
        symbols = renderer.find('symbols').findall('symbol')
        rules = renderer.find('rules').findall('rule')
        rule_dict = dict()
        symbol_dict = dict()
        for s in symbols:
            props = s.find('layer').findall('prop')
            for p in props:
                if p.attrib['k'] == 'color':
                    symbol_dict[s.attrib['name']] = p.attrib['v']
        for r in rules:
            if r.attrib['filter'].lower() == 'else':
                continue
            rule_dict[r.attrib['label']]=[r, r.attrib['filter'], symbol_dict[r.attrib['symbol']]]
            if 'checkstate' in r.attrib:
                rule_dict[r.attrib['label']]+=[int(r.attrib['checkstate'])]
            else:
                rule_dict[r.attrib['label']]+=[1]
        return tree, rule_dict

    def accept(self):
        """Triggered on OK push button"""
        vlayout = self.findChild(QVBoxLayout)
        hlayouts = vlayout.findChildren(QHBoxLayout)
        thresholds = []
        scene_thresholds = []
        for hlayout in hlayouts:
            sbs = [hlayout.itemAt(i).widget() for i in range(hlayout.count()) if isinstance(hlayout.itemAt(i).widget(),QgsDoubleSpinBox)]
            borne_values = sorted([sb.value() for sb in sbs])
            #from dictionnary,get the xml element, the rule label and the rule filter (from both qml files)
            label = hlayout.objectName()
            slabs_rule_class , slabs_filter_class, _, _ = self.__slabs_default_rule_dict[label]
            probe_rule_class , probe_filter_class, _, _ = self.__probe_default_rule_dict[label]
            estimatedblocs_rule_class , estimatedblocs_filter_class, _, _ = self.__estimatedblocs_default_rule_dict[label]
            rule_atlas_class, _, _, _ = self.__atlas_rule_dict[label]
            #then replace label and filter values with the chosen ones.
            if label.startswith('+'):
                slabs_rule_class.set('label',"+{}".format(borne_values[0]))
                probe_rule_class.set('label',"+{}".format(borne_values[0]))
                estimatedblocs_rule_class.set('label',"+{}".format(borne_values[0]))
                rule_atlas_class.set('label',"+{}".format(borne_values[0]))
            else :
                slabs_rule_class.set('label',"{} - {}".format(borne_values[0],borne_values[1]))
                probe_rule_class.set('label',"{} - {}".format(borne_values[0],borne_values[1]))
                estimatedblocs_rule_class.set('label',"{} - {}".format(borne_values[0],borne_values[1]))
                rule_atlas_class.set('label',"{} - {}".format(borne_values[0],borne_values[1]))
            if hlayout.itemAt(0).widget().isChecked():
                slabs_rule_class.set('checkstate',"1")
                probe_rule_class.set('checkstate',"1")
                estimatedblocs_rule_class.set('checkstate',"1")
                thresholds.append(borne_values[0])
                thresholds.append(borne_values[1])
            else:
                slabs_rule_class.set('checkstate',"0")
                probe_rule_class.set('checkstate',"0")
                estimatedblocs_rule_class.set('checkstate',"0")
                thresholds.append(-1)
                thresholds.append(-1)
            slabs_rule_class.set('filter',"""if(  "radiometry_probe" is not NULL,
                                       "radiometry_probe" >= {min} AND "radiometry_probe" <= {max},
                                       "radiometry" >= {min} AND "radiometry" <= {max})
                                    """.format(min="%.6f"%borne_values[0],max="%.6f"%borne_values[1])) #format min and max as a 6 precision float
            probe_rule_class.set('filter', """"radiometry" >= {min} AND "radiometry" <= {max}""".format(min="%.6f"%borne_values[0],max="%.6f"%borne_values[1]))
            estimatedblocs_rule_class.set('filter', """"radiometry" >= {min} AND "radiometry" <= {max}""".format(min="%.6f"%borne_values[0],max="%.6f"%borne_values[1]))
            rule_atlas_class.set('filter',"""if(  "radiometry_probe" is not NULL,
                                       "radiometry_probe" >= {min} AND "radiometry_probe" <= {max},
                                       "radiometry" >= {min} AND "radiometry" <= {max}) AND to_string( "centroid_z") = @atlas_pagename 
                                    """.format(min="%.6f"%borne_values[0],max="%.6f"%borne_values[1]))

            # Update the 3D thresholds
            scene_thresholds.append(borne_values)

        # Update 3d thresholds
        with open(os.path.join(os.path.dirname(__file__), "viewer_3d", "settings_data.py")) as scene_thesholds_stream:
            scene_thesholds_file = scene_thesholds_stream.readlines()
        for idx, i in enumerate([5, 11, 17, 23, 29, 35]):
            scene_thesholds_file[i - 1] = f'        "min": {scene_thresholds[idx][0]},\n'
            scene_thesholds_file[i] = f'        "max": {scene_thresholds[idx][1]},\n'
        with open(os.path.join(os.path.dirname(__file__), "viewer_3d", "settings_data.py"), 'w') as scene_thesholds_stream:
            scene_thesholds_stream.writelines(scene_thesholds_file)

        #write qml output
        self.__slabs_default_tree.write(self.__slabs_defaut_style_path)
        self.__probe_default_tree.write(self.__probe_defaut_style_path)
        self.__estimatedblocs_default_tree.write(self.__estimatedblocs_defaut_style_path)
        self.__atlas_tree.write(self.__atlas_style_path)

        #apply the two qml files in the layer style
        lyr = self.__project.slabsLayer()
        style_manager = lyr.styleManager()
        style_manager.setCurrentStyle('filtre_atlas')
        lyr.loadNamedStyle(self.__atlas_style_path)
        style_manager.setCurrentStyle('défaut')
        lyr.loadNamedStyle(self.__slabs_defaut_style_path)
        self.__project.probeLayer().loadNamedStyle(self.__probe_defaut_style_path)
        self.__project.estimatedBlocsLayer().loadNamedStyle(self.__estimatedblocs_defaut_style_path)
        QgsProject.instance().write()

        #update the truck_loads_status view (quality column depends on the rules)
        self.__data.update_truck_loads_status_view(thresholds)
        super().accept()
